# This file is auto-generated from github.com/arenaxr/arena-schema, changes here may be overwritten.

from .arena_object import *
from .arenaui_button_panel import *
from .arenaui_card import *
from .arenaui_prompt import *
from .box import *
from .camera import *
from .capsule import *
from .circle import *
from .cone import *
from .cylinder import *
from .dodecahedron import *
from .entity import *
from .gaussian_splatting import *
from .gltf_model import *
from .hand_left import *
from .hand_right import *
from .icosahedron import *
from .image import *
from .light import *
from .line import *
from .ocean import *
from .octahedron import *
from .pcd_model import *
from .plane import *
from .ring import *
from .roundedbox import *
from .sphere import *
from .tetrahedron import *
from .text import *
from .thickline import *
from .threejs_scene import *
from .torus import *
from .torus_knot import *
from .triangle import *
from .videosphere import *
from .program import *

OBJECT_TYPE_MAP = {
    Object.object_type: Object,
    ArenauiButtonPanel.object_type: ArenauiButtonPanel,
    ArenauiCard.object_type: ArenauiCard,
    ArenauiPrompt.object_type: ArenauiPrompt,
    Box.object_type: Box,
    Camera.object_type: Camera,
    Capsule.object_type: Capsule,
    Circle.object_type: Circle,
    Cone.object_type: Cone,
    Cylinder.object_type: Cylinder,
    Dodecahedron.object_type: Dodecahedron,
    Entity.object_type: Entity,
    GaussianSplatting.object_type: GaussianSplatting,
    GltfModel.object_type: GltfModel,
    HandLeft.object_type: HandLeft,
    HandRight.object_type: HandRight,
    Icosahedron.object_type: Icosahedron,
    Image.object_type: Image,
    Light.object_type: Light,
    Line.object_type: Line,
    Ocean.object_type: Ocean,
    Octahedron.object_type: Octahedron,
    PcdModel.object_type: PcdModel,
    Plane.object_type: Plane,
    Ring.object_type: Ring,
    Roundedbox.object_type: Roundedbox,
    Sphere.object_type: Sphere,
    Tetrahedron.object_type: Tetrahedron,
    Text.object_type: Text,
    Thickline.object_type: Thickline,
    ThreejsScene.object_type: ThreejsScene,
    Torus.object_type: Torus,
    TorusKnot.object_type: TorusKnot,
    Triangle.object_type: Triangle,
    Videosphere.object_type: Videosphere,
}