# This file is auto-generated from github.com/arenaxr/arena-schema, changes here may be overwritten.

from .translate import *
from .attribute import *
from .color import *
from .data import *
from .animation import *
from .animation_mixer import *
from .armarker import *
from .attribution import *
from .blip import *
from .box_collision_listener import *
from .click_listener import *
from .dynamic_body import *
from .gltf_model_lod import *
from .gltf_morph import *
from .goto_landmark import *
from .goto_url import *
from .impulse import *
from .jitsi_video import *
from .landmark import *
from .material import *
from .material_extras import *
from .multisrc import *
from .position import *
from .remote_render import *
from .rotation import *
from .scale import *
from .shadow import *
from .sound import *
from .spe_particles import *
from .static_body import *
from .textinput import *
from .video_control import *

ATTRIBUTE_TYPE_MAP = {
    "attribute": Attribute,
    "color": Color,
    "data": Data,
    "animation": Animation,
    "animation_mixer": AnimationMixer,
    "armarker": Armarker,
    "attribution": Attribution,
    "blip": Blip,
    "box_collision_listener": BoxCollisionListener,
    "click_listener": ClickListener,
    "dynamic_body": DynamicBody,
    "gltf_model_lod": GltfModelLod,
    "gltf_morph": GltfMorph,
    "goto_landmark": GotoLandmark,
    "goto_url": GotoUrl,
    "impulse": Impulse,
    "jitsi_video": JitsiVideo,
    "landmark": Landmark,
    "material": Material,
    "material_extras": MaterialExtras,
    "multisrc": Multisrc,
    "position": Position,
    "remote_render": RemoteRender,
    "rotation": Rotation,
    "scale": Scale,
    "shadow": Shadow,
    "sound": Sound,
    "spe_particles": SpeParticles,
    "static_body": StaticBody,
    "textinput": Textinput,
    "video_control": VideoControl,
}
